/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoPointRTAMap = new Array();


//========================================================================================================
// class : EPColorChooser
//
// Encapsulates the RTA global color chooser
//========================================================================================================
function EPColorChooser() {
	this.htmlId = 'eprtaColorChooserTable';
	this.targetF = null;
}

// one and only ColorChooser!
var EchoPointRTACC = new EPColorChooser();

//------------------------------------------------------------
// This method sets the specified 'f' function as the target for color
// notifications.  The function must be in the form
//
//			function(newClr)
//
//------------------------------------------------------------
EPColorChooser.prototype.setTarget = function(f) {
	this.targetF = f;
}

//------------------------------------------------------------
// Hides the RTA Color Chooser 
//------------------------------------------------------------
EPColorChooser.prototype.hide = function() {
	var e = getElementById(this.htmlId);
	if (e == null) {
		alert("ASSERT : color chooser is not present");
		return;
	}
	e.setDisplayed(false);
}

//------------------------------------------------------------
// Shows the RTA Color Chooser next to the specified element
//------------------------------------------------------------
EPColorChooser.prototype.showNextTo = function(nextToHtmlId) {
	var e = getElementById(this.htmlId);
	if (e == null) {
		alert("ASSERT : color chooser is not present");
		return;
	}
	var x = 100;
	var y = 100;
	if (nextToHtmlId != null) {
		var nextToE = getElementById(nextToHtmlId);
		x = nextToE.getX();
		y = nextToE.getY() + nextToE.getHeight();
	}
	e.setX(x);
	e.setY(y);
	e.setDisplayed(true);
}

//------------------------------------------------------------
// Can be called to get the global color chooser allowing
// you to hide it or show it.
//------------------------------------------------------------
function g_eprtaGetColorChooser() {
	if (EchoPointRTACC == null)
		EchoPointRTACC = new EPColorChooser();
	return EchoPointRTACC;
}


//------------------------------------------------------------
//
// Called by the RTA ColorChooser table as the user moves over 
// the TDs within it or click on it.
//
//------------------------------------------------------------
function g_eprtaCCEvent(event, td, eventtype) {
	var ev = browser.isIE ? window.event : event;

	if (eventtype == "mouseover") {
		td.style.border='1px dotted white';
	}
	if (eventtype == "mouseout") {
		td.style.border='1px solid gray';
	}
	if (eventtype == "click") {
		//epbe_CancelEvent(ev);

		var gCC = g_eprtaGetColorChooser();
		if (gCC.targetF != null) {
			var clr = td.id;
			//if (clr.indexOf("#") == 0)
			//	clr = clr.substring(1);
			gCC.targetF(clr);
		}
		gCC.hide();
	}
}

//========================================================================================================
// class : EPTBLandF
//
// The encapsulates the styles that must be applied to a ToolBar object
//========================================================================================================
function EPTBLandF(upItemClass, downItemClass, upMouseOverClass, downMouseOverClass) {
	this.upItemClass = upItemClass;
	this.downItemClass = downItemClass;
	this.upMouseOverClass = upMouseOverClass;
	this.downMouseOverClass = downMouseOverClass;
}


//========================================================================================================
// class : EPToolBarItem
//
// A class that encapsulates a tool bar button or select field.  An 'isStateful' item may have 2 
// states, "up" or "down", in which case the 'pressedState' property will be either false or true. If its
// not isStateful, then it is always "up"
//
//========================================================================================================
function EPToolBarItem(rta, htmlId, cmd, isStateful, landf) {
	this.rta = rta;
	this.htmlId = htmlId;
	this.cmd	= cmd;
	this.isStateful = isStateful;
	this.landf = landf;
	this.pressedState = false;
	this.e = null;
	this.elem = null;
}

//------------------------------------------------------------
// Delayed construction of toolbar item
//------------------------------------------------------------
EPToolBarItem.prototype.create = function() {
	this.e = getElementById(this.htmlId);
	if (this.e == null)
		alert('ASSERT : TBItem htmlId is invalid!');

	this.elem = this.e.htmlE;
	this.elem.className = this.landf.upItemClass;

	var tag = this.elem.tagName.toLowerCase();
	if (tag == "img") {
		epbe_AddEventHandler(this, this.elem, window, "mouseover");
		epbe_AddEventHandler(this, this.elem, window, "mouseout");
		epbe_AddEventHandler(this, this.elem, window, "mousedown");
		epbe_AddEventHandler(this, this.elem, window, "mouseup");
		epbe_AddEventHandler(this, this.elem, window, "click");
	}
	if (tag == "select") {
		epbe_AddEventHandler(this, this.elem, window, "change");
	}
}

//------------------------------------------------------------
// Called to execute the command associatde with the given
// TB item.
//------------------------------------------------------------
EPToolBarItem.prototype.execCommand = function(value,showUI) {
	this.rta.execCommand(this.cmd,value,showUI);
}

//------------------------------------------------------------
// This is the generic event handler for the TBItem.  All events
// are channelled to this function.  
//------------------------------------------------------------
EPToolBarItem.prototype.eventHandler = function(ev) {
	var elem = this.elem;
	var tag = this.elem.tagName.toLowerCase();
	//
	// button control
	if (ev.type == "click" && tag == "img") {
		//
		// handle special cases here
		var tbi = this;
		if (this.cmd == "hilitecolor" || this.cmd == "forecolor") {
			var f = function(clr) {
				tbi.execCommand(clr);
			}
			g_eprtaGetColorChooser().setTarget(f);
			g_eprtaGetColorChooser().showNextTo(this.htmlId);
		} 
		else if (this.cmd == "createlink" || this.cmd == "insertimage") {
			if (browser.isIE) {
				this.execCommand(null,true);
			} else {
				if (url) {
					this.execCommand(url);
				}
						}	
		} else {
			this.execCommand(null);
		}
		return;
	}
	//
	// select control
	if (ev.type == "change" && tag == "select") {
		var value = elem.options[elem.selectedIndex].value;
		//
		// we only add the <> brackets when needed!
		if (this.cmd == 'formatblock')
			value = '<' + value + '>';

		this.execCommand(value);
		return;
	}
	//
	// handle mouse over/out/down events for button appearance reasons. We use the various style classes
	// to control how it looks at mouse over times
	//
	if (ev.type.indexOf("mouse") == 0 && tag == "img") {
		this.redrawButtonAppearance(ev.type);
		return;
	}
}

//------------------------------------------------------------
// Called to update the state of the toolbar item, ie make it 
// reflect the current value in either button or a select 
// field.
//------------------------------------------------------------
EPToolBarItem.prototype.updateState = function() {
	var i = 0;
	var tag = this.elem.tagName.toLowerCase();
	var cmd = this.cmd;

	// 
	// only buttons that are stateful get updated
	if (tag == "img" && this.isStateful) {
		this.pressedState = false;
		if (this.rta.queryCommandEnabled(cmd)) {
			this.pressedState = this.rta.queryCommandState(cmd);	
		}
		this.redrawButtonAppearance("mouseout");
	}
	if (tag == "select") {
		var select = this.elem;
		var foundVal = false;
		var value = this.rta.queryCommandValue(cmd);
		if (value && select.options) {
			curVal = (""+value).toLowerCase();
			// try and find a select option that matches that value
			for (i = 0; i < select.options.length; i++) {
				var opval = (""+ select.options[i].value).toLowerCase();
				var opSubVal = ( curVal.length <= opval.length ? opval : opval.substr(0, curVal.length));
				if (opval == curVal || opSubVal == curVal) {
					select.selectedIndex = i;
					foundVal = true;
					break;
				}
			}
			if (! foundVal) {
				//
				// no good!  try and find a select option that matches option text
				for (i = 0; i < select.options.length; i++) {
					var opval = (""+ select.options[i].text).toLowerCase();
					var opSubVal = ( curVal.length <= opval.length ? opval : opval.substr(0, curVal.length));
					if (opval == curVal || opSubVal == curVal) {
						select.selectedIndex = i;
						foundVal = true;
						break;
					}
				}
			}
		}
		if (! foundVal) {
			//
			// we didnt find one so set the selected index to the last one
			// in the list.  This is defined to be an "indeterminate" value
			select.selectedIndex = select.options.length-1;
		}
	}
}


//------------------------------------------------------------
// Called to force a tbItem to redraw its appearance based on
// an specific event happening such as mouseout
//------------------------------------------------------------
EPToolBarItem.prototype.redrawButtonAppearance = function(eventType) {
	var elem = this.elem;

	if (eventType == "mouseover") {
		elem.className = (this.pressedState) ? this.landf.downMouseOverClass : this.landf.upMouseOverClass;
	}
	if (eventType == "mouseout") {
		elem.className = (this.pressedState) ? this.landf.downItemClass : this.landf.upItemClass;
	}
	//
	// cause the images to "jump" down a little bit on click
	if (eventType == "mousedown") {
		elem.style.left = 1;
		elem.style.top  = 1;
	}
	else {
		elem.style.left = 0;
		elem.style.top  = 0;
	}
}


//========================================================================================================
// class : EPRTA
//
// Our top level RichText class
//========================================================================================================
function EPRTA(htmlId) {
	this.htmlId = htmlId;
	this.rtaDoc = null;
	this.rtaWin = null;

	this.toolbarItems = new Array();
	this.currentHTML = null;
	EchoPointRTAMap[htmlId] = this;
}

//------------------------------------------------------------
// Delayed load of RTA features.  Allows for early object 
// construction then later document load  finishing
//------------------------------------------------------------
EPRTA.prototype.create = function(rtaDoc,rtaWin,initialHTML) {
	this.rtaDoc = rtaDoc;
	this.rtaWin = rtaWin;
	this.currentHTML = initialHTML;

	//	
	// single event handler handles multiple events
	epbe_AddEventHandler(this, rtaDoc, rtaWin, "keydown");
	epbe_AddEventHandler(this, rtaDoc, rtaWin, "keypress");
	epbe_AddEventHandler(this, rtaDoc, rtaWin, "click");
	epbe_AddEventHandler(this, rtaDoc, rtaWin, "focus");

	//
	// now create all our previously registered toolbar items
	for (tbItem in 	this.toolbarItems) {
		this.toolbarItems[tbItem].create();
	}
}

//------------------------------------------------------------
// Called to allow the RTA to save its own state to EP
// Does it smartly so that it doesnt raise events
// unecessarily
//------------------------------------------------------------
EPRTA.prototype.saveState = function() {
	var state = this.rtaDoc.body.innerHTML;
	if (state != this.currentHTML) {
		this.currentHTML = state;
		E_setParameter(this.htmlId, state);
	}
}

//------------------------------------------------------------
// Runs the MISAS/MSHTML queryCommandState
//------------------------------------------------------------
EPRTA.prototype.queryCommandState = function(cmd) {
	return this.rtaDoc.queryCommandState(cmd);
}

//------------------------------------------------------------
// Runs the MISAS/MSHTML queryCommandValue
//------------------------------------------------------------
EPRTA.prototype.queryCommandValue = function(cmd) {
	return this.rtaDoc.queryCommandValue(cmd);
}

//------------------------------------------------------------
// Runs the MISAS/MSHTML queryCommandEnabled
//------------------------------------------------------------
EPRTA.prototype.queryCommandEnabled = function(cmd) {
	return this.rtaDoc.queryCommandEnabled(cmd);
}

//------------------------------------------------------------
// Executes a MIDAS/MSHTML command against the RTA
//------------------------------------------------------------
EPRTA.prototype.execCommand = function(cmd,value,showUI) {
	this.focus();
	cmd = this.remapCommand(cmd);
	if (showUI)
		 this.rtaDoc.execCommand(cmd,showUI,value);
	else
		 this.rtaDoc.execCommand(cmd,false,value);

	this.updateState();
	this.focus();
}

//------------------------------------------------------------
// Remaps the MIDAS/MSHTML command to take into account browser differences
//------------------------------------------------------------
EPRTA.prototype.remapCommand = function(cmd) {
	if (cmd == "hilitecolor" && browser.isIE)
		return "backcolor";

	return cmd;
}


//------------------------------------------------------------
// sets focus to the RTA
//------------------------------------------------------------
EPRTA.prototype.focus = function() {
	this.rtaWin.focus();
}
//------------------------------------------------------------
// updates the RTA state and its toolbars etc.  Called
// on a "delay" so it happens after and event happens
//------------------------------------------------------------
EPRTA.prototype.updateState = function() {
	//
	// the update is done on a small time delay to allow keys to get through
	// and be processed and hence have a more accurate state of play
	//
	window.setTimeout("g_eprtaUpdateToolBar('" + this.htmlId + "');", 10);
}


//------------------------------------------------------------
// Finds the toolbar item with the givcen command or null if it 
// cannot be found.
//------------------------------------------------------------
EPRTA.prototype.findToolBarItem = function(cmd) {
	for (tbItem in 	this.toolbarItems) {
		var tbi = this.toolbarItems[tbItem];
		if (tbi.cmd == cmd)
			return tbi;
	}
	return null;
}

//------------------------------------------------------------
// Called to update the state of each tool bar item that is 
// registered.  It asks the rtaDoc whether the specific command 
// is current given the current position.
//------------------------------------------------------------
EPRTA.prototype.updateToolBar = function() {
	for (tbItem in 	this.toolbarItems) {
		this.toolbarItems[tbItem].updateState();
	}
	// and save the state of control
	this.saveState();
}

//------------------------------------------------------------
// This is the generic event handler for the RTA.  All events
// are channelled to this function.  Therefore it needs to
// distinguish between them.
//------------------------------------------------------------
EPRTA.prototype.eventHandler = function(ev) {
	if (ev.type == "keypress" || ev.type == "keydown") {
		this.onkey(ev);
	}
	if (ev.type == "focus") {
		this.onfocus(ev);	
	}
	g_eprtaGetColorChooser().hide();
	this.updateState();
}

//------------------------------------------------------------
// called when focus is set to the RTA
//------------------------------------------------------------
EPRTA.prototype.onfocus = function(ev) {
}

//------------------------------------------------------------
// called when key enters the RTA
//------------------------------------------------------------
EPRTA.prototype.onkey = function(ev) {
	if (ev.ctrlKey) {
		var cmd = null;
		var key = String.fromCharCode(browser.isIE ? ev.keyCode : ev.charCode).toLowerCase();	
		if (key == 'b')
			cmd = "bold";
		else if (key == 'i')
			cmd = "italic";
		else if (key == 'u')
			cmd = "underline";
		else if (key == 'l')
			cmd = "justifyleft";
		else if (key == 'r')
			cmd = "justifyright";
		else if (key == 'e')
			cmd = "justifycenter";
		else if (key == 'j')
			cmd = "justifyfull";
		else if (key == '=')
			cmd = "subscript";
		else if (key == '-')
			cmd = "superscript";
			
		if (cmd) {
			var tbi = this.findToolBarItem(cmd);
			if (tbi) {
				tbi.execCommand(null);
				epbe_CancelEvent(ev);
			}
		}
	}
}


//------------------------------------------------------------
// Registers a tool bar button item with the RTA and sets its command
// that it will reported back when pressed
//------------------------------------------------------------
EPRTA.prototype.registerButton = function(cmd, isStateful, landf) {
	var itemHtmlId = this.htmlId+'_'+cmd;
	var tbi = new EPToolBarItem(this,itemHtmlId,cmd,isStateful,landf);
	this.toolbarItems[this.toolbarItems.length] = tbi;
	return tbi;
}

//------------------------------------------------------------
// Registers a tool bar button item with the RTA and sets its command
// that it will reported back when pressed
//------------------------------------------------------------
EPRTA.prototype.registerSelect = function(cmd,landf) {
	var itemHtmlId = this.htmlId+'_'+cmd;
	var tbi = new EPToolBarItem(this,itemHtmlId,cmd,false,landf);
	this.toolbarItems[this.toolbarItems.length] = tbi;
	return tbi;
}

//------------------------------------------------------------
// Returns the current selection
//------------------------------------------------------------
EPRTA.prototype.getSelection = function() {
	if (browser.isIE) {
		return this.rtaDoc.selection;
	} else {
		return rtaWindow.getSelection();
	}
};

//------------------------------------------------------------
// returns a range for the provided selection
//------------------------------------------------------------
EPRTA.prototype.createRange = function(selection) {
	if (browser.isIE) {
		return selection.createRange();
	} else {
		this.focus();
		if (typeof selection != "undefined") {
			return selection.getRangeAt(0);
		} else {
			return rtaDoc.createRange();
		}
	}
};


//========================================================================================================
// Global functions
//========================================================================================================

//------------------------------------------------------------
// Called in a time delay to update the tool bar buttons
// for a given RTA.
//------------------------------------------------------------
function g_eprtaUpdateToolBar(htmlId) {
	var rta = EchoPointRTAMap[htmlId];
	if (rta == null) {
		alert('ASSERT : EPRTA object not found! ' + htmlId);
		return;
	}
	rta.updateToolBar();
}




//------------------------------------------------------------
// Creates a rich text object for a given htmlId
//------------------------------------------------------------
function eprtaCreateRTA(htmlId, initialCss, initialText) {
	var rtaDoc = document.getElementById(htmlId).contentWindow.document;

	rtaDoc.designMode = "On";

	var html = "";
	html += "<html>\n";
	html += "<head>\n";
	html += "<style>" + initialCss + "</style>\n";
	html += "</head>\n";
	html += "<body>\n";
	html += initialText;
	html += "</body>\n";
	html += "</html>";

	rtaDoc.open();
	rtaDoc.write(html);
	rtaDoc.close();
	if (browser.isIE) {
		rtaDoc.body.contentEditable = true;
	}

	var rtaWin = document.getElementById(htmlId).contentWindow;
	//
	// get a previously created EPRTA object for load it up with
	// the new values.  This implies that it was 
	var rta = EchoPointRTAMap[htmlId];
	if (rta == null) {
		alert('ASSERT : EPRTA object not found! ' + htmlId);
		return null;
	}
	rta.create(rtaDoc,rtaWin,initialText);
	
	return rta;
}

//------------------------------------------------------------
// Called in the onload script to complete the creation of
// an RTA.  
//------------------------------------------------------------
function g_eprtaOnLoad(htmlId,initialCss,initialText) {
	var rta = eprtaCreateRTA(htmlId,initialCss,initialText);
	try {
		rta.execCommand("bold", true);
		rta.execCommand("bold", false);
	}  catch (e) {
		alert("ASSERT : Rich Text Editing is not supported on this browser version.");
	}	
	rta.updateState();
}

function dbg(s) {
	var e = getElementById('dbg');
	if (e) {
		var save = e.htmlE.innerHTML;
		e.htmlE.innerHTML = s + '\n' + save;
	}
}