/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.rpc.impl;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.Dictionary;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;
import org.soafaces.bundle.client.ui.Weblet;
import org.soafaces.bundle.client.ui.BaseIOViewer;
import org.soafaces.bundle.client.ui.InputViewer;
import org.soafaces.bundle.client.ui.OutputViewer;
import org.soafaces.bundle.client.ui.RuntimeViewer;

/**
 * 
 * @author Sam Taha
 */
public class ApplicationBoot {
  
  public static final String WEBLET="weblet";
  public static final String INPUTVIEWER="inputviewer";
  public static final String OUTPUTVIEWER="outputviewer";
  public static final String RUNTIMEVIEWER="runtimeviewer";
  
  private static int _iAppJobType=0; //0 is app, 1 is job
  private static String _stUsername=null;
  private static String _stUIType=null;
  private static long _idJob = -1;
  private static long _idRun = -1;
  private static long _idTask = -1;
  //private static int _iTaskletRetry=0; //0 false, 1 true
  private static int _iMode = -1;
  private static long _lEffDate = -1;
  private static String _stTaskPackageName=null;
  private static String _stContext = null;
  private static BundleDataAccess _oTBAccess = null;
  private static boolean _bIsEntryPointLoaded=false;
  private static boolean _bServicesLoaded=false;
  
  public void createApplication(EntryPoint entrypoint) {   
    
    //Entrypoints are called one after the other in order they are
    //displayed in the gwt.xml file. With this condition checking it
    //insures that one and only one Entrypoint is called
    if(isEntryPointLoaded())
      return;
    
    if(!_bServicesLoaded)
      loadServices();
       
    Object oEditorViewer = entrypoint;
    //Object oEditorViewer = null; //new surda.beantest.demogwt.client.DemoGWTCustomizer();

    //order of this if/else is important
    Panel mainPanel = null;

    try {
      if(_stUIType.equals(WEBLET) && entrypoint instanceof Weblet)
      {
        Weblet.Mode realMode = MapWebletViewerEnums.getWebletEnum(_iMode);
        _oTBAccess.associateCustomizer((Weblet)oEditorViewer);
        ((Weblet)oEditorViewer).initUI(_oTBAccess, realMode);
        mainPanel = ((Weblet)oEditorViewer).getMainPanel();
        _bIsEntryPointLoaded=true;

      }
      else if(_stUIType.equals(RUNTIMEVIEWER) && entrypoint instanceof RuntimeViewer)
      {
        RuntimeViewer.Mode realMode = MapWebletViewerEnums.getRuntimeViewerEnum(_iMode);
        ((RuntimeViewer)oEditorViewer).initUI(_oTBAccess, realMode);
        mainPanel = ((RuntimeViewer)oEditorViewer).getMainPanel();
        _bIsEntryPointLoaded=true;
      }
      else if(_stUIType.equals(INPUTVIEWER) && entrypoint instanceof InputViewer)
      {
        BaseIOViewer.Mode realMode = MapWebletViewerEnums.getViewerEnum(_iMode);
        ((BaseIOViewer)oEditorViewer).initUI(_oTBAccess, realMode);
        mainPanel = ((BaseIOViewer)oEditorViewer).getMainPanel();
        _bIsEntryPointLoaded=true;
      }
      else if(_stUIType.equals(OUTPUTVIEWER) && entrypoint instanceof OutputViewer)
      {
        BaseIOViewer.Mode realMode = MapWebletViewerEnums.getViewerEnum(_iMode);
        ((BaseIOViewer)oEditorViewer).initUI(_oTBAccess, realMode);
        mainPanel = ((BaseIOViewer)oEditorViewer).getMainPanel();
        _bIsEntryPointLoaded=true;
      }
      else
      {
        //Window.alert("Unknown module type: " + _stUIType);
        return;
      }

      if(mainPanel != null && mainPanel != RootPanel.get())
        RootPanel.get().add(mainPanel);

      Panel pSpinner = RootPanel.get("go-loadingdiv");
      if(pSpinner != null)
        pSpinner.setVisible(false);
      
    }catch(Throwable ex) {
      Window.alert("sofaces loading error: " + ex.getMessage());
      return;
    }
  }
  
  private void loadServices()
  {
    _bServicesLoaded=true;
    
    GWT.setUncaughtExceptionHandler(new GWT.UncaughtExceptionHandler() {
      public void onUncaughtException(Throwable e) {
        Window.alert("soafaces error: App Runtime Error - " + e.getMessage());
      }
    });
    
    jsInitOne();
    jsInitTwo();
    
    //Read params
    Dictionary appParams = Dictionary.getDictionary("SOAFacesAppParams");
    
    _stUIType = appParams.get("uitype");

    //_iTaskletRetry = Integer.parseInt(appParams.get("retry"));
    _iAppJobType = Integer.parseInt(appParams.get("appjobtype"));
    _idJob = Long.parseLong(appParams.get("jobid"));
    _idRun = Long.parseLong(appParams.get("runid"));
    _idTask = Long.parseLong(appParams.get("taskid"));
    _iMode = Integer.parseInt(appParams.get("mode"));
    _lEffDate = Long.parseLong(appParams.get("effdate")); //long number or -1 if not set
    _stTaskPackageName = appParams.get("tasktype"); //eg: org-acme-beans-MyBean-v1-0-0
    _stContext = appParams.get("context");
    BundleUniversalClient bundleClient = new BundleUniversalClient(_idJob, _idTask, _stContext, _stTaskPackageName);

    boolean bIsApp=true;
    if(_iAppJobType == 1)
      bIsApp=false;

    //boolean bIsRetryTasklet=false;
    //if(_iTaskletRetry == 1)
    //  bIsRetryTasklet=true;
    
    _oTBAccess = new BundleDataAccess (_stContext, 
                                       _iMode, _idJob, _idRun, _idTask, bIsApp,
                                       _lEffDate, _stUIType, _stUsername,
                                       _stTaskPackageName, bundleClient);
  }
  
  private static boolean isEntryPointLoaded() { return _bIsEntryPointLoaded; }
  
  /**
   * This is called by javascript. Check the native method <code>jsInit()</code>
   * 
   * @return 0 if ok, -1 if there is a server side problem, -2 validation error
   */
  public static int saveInputBean() {
    return _oTBAccess.saveInputBeanInternal();
  }
  
  /**
   * This is called by javascript. Check the native method <code>jsInit()</code>
   * 
   * @return 0 if ok, -1 if there is a server side problem, -2 validation error
   */
  public static int saveCacheInputBean() {
    return _oTBAccess.saveCacheInputBean();
  }

  /**
   * Called when this class is initialized. Sets a javascript function variable
   * to reference a static method of an object.
   */
  private static native void jsInitOne() /*-{
     $wnd.SOAFacesApplication_saveInputBean = function()
     {
       //$wnd.alert('Really about to call saveInputBean');
       return @org.soafaces.bundle.client.rpc.impl.ApplicationBoot::saveInputBean()();
     }
   }-*/;

  /**
   * Called when this class is initialized. Sets a javascript function variable
   * to reference a static method of an object.
   */
  private static native void jsInitTwo() /*-{
     $wnd.SOAFacesApplication_saveCacheInputBean = function()
     {
       //$wnd.alert('Really about to call saveCacheInputBean');
       return @org.soafaces.bundle.client.rpc.impl.ApplicationBoot::saveCacheInputBean()();
     }
  }-*/;

  /**
   * Pass control back to web container to save and change screens.
   */
  static native void echoSaveAndReturn() /*-{
     $wnd.parent.saveAndReturn();
  }-*/;
  
  /**
   * Pass control back to container to execute app and move to next step.
   */
  static native void echoAppExecuteAndReturn() /*-{
     $wnd.parent.appExecuteAndReturn();
  }-*/;
  
}