/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.rpc.impl;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import java.io.Serializable;
import java.util.Map;
import org.soafaces.services.client.rpc.UniversalClient;
import org.soafaces.services.client.rpc.UniversalClientJSONCallback;
import org.soafaces.services.client.rpc.UniversalClientPOJOCallback;
import org.soafaces.services.client.rpc.GenericDataWrapper;
import org.soafaces.services.client.rpc.GenericDataUtil;


/**
 *
 * @author Sam Taha
 */
class BundleUniversalClient implements UniversalClient {

  private BundleUniversalClientRPCAsync _oBundleClientService = null;
  private String _stContextName = null;
  private String _stTaskType = null;
  private long _lJobID = -1;
  private long _lTaskID = -1;
  
  private BundleUniversalClient() {}

  public BundleUniversalClient(long lJobID, long lTaskID, String contextName, String stTaskType) //, int mode, String stTaskType,long idJob, long idRun, long idTask) 
  {
    _stTaskType = stTaskType;
    _stContextName = contextName; //should be just "jobserver"
    _oBundleClientService = (BundleUniversalClientRPCAsync) GWT.create(BundleUniversalClientRPC.class);
    _lJobID = lJobID;
    _lTaskID = lTaskID;

    ServiceDefTarget endpointMuleClient = (ServiceDefTarget) _oBundleClientService;
    String muleModuleRelativeURL = "/" + _stContextName + "/rpcservices/muleclientservice?tasktype=" +_stTaskType;
    endpointMuleClient.setServiceEntryPoint(muleModuleRelativeURL);
  }

  public void send(String endpointURI, Object payload, Map<String, Serializable> properties, UniversalClientPOJOCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oBundleClientService.sendAnyReturnPOJO(_lJobID, _lTaskID, endpointURI, serializablePayload, properties, callback);
  }

  public void send(String endpointURI, Object payload, UniversalClientPOJOCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oBundleClientService.sendAnyReturnPOJO(_lJobID, _lTaskID, endpointURI, serializablePayload, null, callback);
  }
   
  public void send(String endpointURI, UniversalClientPOJOCallback callback) {
    _oBundleClientService.sendEmptyReturnPOJO(_lJobID, _lTaskID, endpointURI, null, callback);
  }

  public void send(String endpointURI, Object payload, Map<String, Serializable> properties, UniversalClientJSONCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oBundleClientService.sendAnyReturnJSON(_lJobID, _lTaskID, endpointURI, serializablePayload, properties, callback);
  }

  public void send(String endpointURI, Object payload, UniversalClientJSONCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oBundleClientService.sendAnyReturnJSON(_lJobID, _lTaskID, endpointURI, serializablePayload, null, callback);
  }
  
  public void send(String endpointURI, UniversalClientJSONCallback callback) {
    _oBundleClientService.sendEmptyReturnJSON(_lJobID, _lTaskID, endpointURI, null, callback);
  }

  public void dispatch(String endpointURI, Object payload, Map<String, Serializable> properties, AsyncCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);

    _oBundleClientService.dispatchAny(_lJobID, _lTaskID, endpointURI, serializablePayload, properties, callback);
  }

  public void dispatch(String endpointURI, Object payload, AsyncCallback callback) {
    GenericDataWrapper serializablePayload = GenericDataUtil.createGenericDataWrapper(payload);
    
    _oBundleClientService.dispatchAny(_lJobID, _lTaskID, endpointURI, serializablePayload, null, callback);
  }

  public void dispatch(String endpointURI, AsyncCallback callback) {
    _oBundleClientService.dispatchEmpty(_lJobID, _lTaskID, endpointURI, null, callback);
  }
}
